/*
 * Copyright 2003-2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.codehaus.groovy.classgen

/**
 * @author Guillaume Laforge
 */
class CallClosureFieldAsMethodTest extends GroovyTestCase {

    String firstname = "Guillaume"
    def closureMethod = { greeting-> "${greeting} ${firstname}" }
    public static staticClosureMethod = {it}

    /**
     * Check that we can call a closure defined as a field as if it were a normal method
     */
    void testCallToClosureAsMethod() {
        def obj = new CallClosureFieldAsMethodTest()
        assert obj.closureMethod("Hello") == "Hello Guillaume"
    }
    
    void testCallToClosureAsMethodFromStaticField() {
        assert CallClosureFieldAsMethodTest.staticClosureMethod("Hello") == "Hello"
    }
    
    void testEnsureCallMethodIsUsed() {
      assertScript """
        class Dummy {
            def call(Object arguments) {"1"}
        }
        def c = new Dummy()
        assert c(2) == "1"      
      """    
    }

}
